#include "funktional.h"

// ##########################################################################

Funktional::Funktional() : zaehler(0)
{}

Funktional::~Funktional()
{
#ifdef DEBUG
  cout << "Funktional::~Funktional()" << endl;
#endif
  initialize();
}

void Funktional::initialize() 
{
  for (FList::iterator it = f_liste.begin(); it != f_liste.end(); it++)
    delete *it;
  f_liste.erase(f_liste.begin(), f_liste.end());
  zaehler = 0;
#ifdef DEBUG
  cout << "Funktional::initialize()" << endl;
#endif
}

int Funktional::additem(FunktionalItem *item) 
{		
  f_liste.push_back(item);
  item->set_ziel(zaehler);
  return(zaehler++);
}

double Funktional::execute(double a, double b, vector<double> &y)
{
  int ylength = y.size() - 1;
  if (!zaehler)
    return 0;
#ifdef DEBUG
  cout << "Bart Simpson:..." << endl;
#endif
  ergebnis_array.reserve(zaehler);
  FunktionalItem::ExecParam param(a, b, y, ergebnis_array);
  
  double rvalue = 0;

  
  for (FList::iterator it = f_liste.begin(); it != f_liste.end(); it++) {
    (*it)->execute_item(0, param);
  }
  rvalue += ergebnis_array[zaehler-1];
  
  for (FList::iterator it = f_liste.begin(); it != f_liste.end(); it++) {
    (*it)->execute_item(ylength, param);
  }
  rvalue += ergebnis_array[zaehler-1];
  
  // Diese for-Schleife integriert mit der Sample-Tiefe leny
  double j = 4;
  for (int i = 1; i < ylength; i++) {
    // Fuer jeden x-Wert berechnen wir den Wert des Funktionals
    for (FList::iterator it = f_liste.begin(); it != f_liste.end(); it++) {
      (*it)->execute_item(i, param);
    }
    // multipliziere abwechselnd mit 2 und mit 4
    rvalue += ergebnis_array[zaehler-1] * j;
    j = 6-j; 
  }
  // Fr die Simpson-Formel sind nur 
  // ungeradzahlige Sampletiefen zugelassen, daher mogeln wir den
  // letzten Wert gegebenenfalls dazu
  if (j == 4) {
    rvalue += ergebnis_array[zaehler-1] * j;
    rvalue *= (b-a)/(3*(ylength + 1));
  } else {
    rvalue *= (b-a)/(3*ylength);
  }

#if 0

  for (FList::iterator it = f_liste.begin(); it != f_liste.end(); it++) {
    (*it)->execute_item(0, param);
  }
  double rvalue_alt = 2*ergebnis_array[zaehler-1];

  for (int i = 1; i < ylength; i++) {
    // Fuer jeden x-Wert berechnen wir den Wert des Funktionals
    for (FList::iterator it = f_liste.begin(); it != f_liste.end(); it++) {
      (*it)->execute_item(i, param);
    }
    rvalue += rvalue_alt + (ergebnis_array[zaehler-1] - rvalue_alt)/2;
    rvalue_alt = ergebnis_array[zaehler-1];
  }

  for (FList::iterator it = f_liste.begin(); it != f_liste.end(); it++) {
	  (*it)->execute_item(ylength, param);
  }
  rvalue += 2*(rvalue_alt + (ergebnis_array[zaehler-1] - rvalue_alt)/2);

  rvalue *= (b-a)/(ylength);
#endif
#ifdef DEBUG
  cout << "Homer Simpson: " << rvalue << endl;
#endif
  return rvalue;
}

#include "solve_parser.tab.c"

char * Funktional::parse(char *str)
{
  parse_string = (unsigned char *)str;
  parse_position = 0;
  parse_error = NULL;
 
  initialize();
  
  // Nicht gerade Thread-safe, aber was will man machen (auer Spinlocks
  // ich wei)
  
  scanned_funktional = this;
  yyparse();
  return (char *) parse_error;
}

// ##########################################################################

FunktionalItem::~FunktionalItem() {}

// ##########################################################################

void fi_konst::execute_item(int, ExecParam & param) {
  param.ergebnis_array[ergebnis_ziel] = zahl;
}

void fi_add::execute_item(int, ExecParam & param) {
	param.ergebnis_array[ergebnis_ziel] = 
		param.ergebnis_array[zahl1] 
		+ param.ergebnis_array[zahl2];
}

void fi_sub::execute_item(int, ExecParam & param) {
	param.ergebnis_array[ergebnis_ziel] = 
		param.ergebnis_array[zahl1] 
		- param.ergebnis_array[zahl2];
}

void fi_mul::execute_item(int, ExecParam & param) {
	param.ergebnis_array[ergebnis_ziel] = 
		param.ergebnis_array[zahl1] *
		param.ergebnis_array[zahl2];
}

void fi_div::execute_item(int, ExecParam & param) {
	if (!param.ergebnis_array[zahl2]) {
		// Fuer Physiker ist das unendlich...
		if (param.ergebnis_array[zahl1] > 0) {
			param.ergebnis_array[ergebnis_ziel] = HUGE_VAL;
		} else if (param.ergebnis_array[zahl1] == 0) {
				// Am liebsten wuerde ich jetzt srand nehmen...
			param.ergebnis_array[ergebnis_ziel] = 0;
		} else {
			param.ergebnis_array[ergebnis_ziel] =-HUGE_VAL;
		}
	} else {
		param.ergebnis_array[ergebnis_ziel] = 
			param.ergebnis_array[zahl1] /
			param.ergebnis_array[zahl2];
	}
}

void fi_sin::execute_item(int, ExecParam & param) {
	param.ergebnis_array[ergebnis_ziel] = sin(param.ergebnis_array[zahl]);
}
void fi_cos::execute_item(int, ExecParam & param) {
	param.ergebnis_array[ergebnis_ziel] = cos(param.ergebnis_array[zahl]);
}

void fi_tan::execute_item(int, ExecParam & param) {	
	// FIXME! Auf Vielfache von \pi/2 berprfen!
	param.ergebnis_array[ergebnis_ziel] = 
		tan(param.ergebnis_array[zahl]);
}

void fi_exp::execute_item(int, ExecParam & param) {
	param.ergebnis_array[ergebnis_ziel] = 
		exp(param.ergebnis_array[zahl]);
}

void fi_ln::execute_item(int, ExecParam & param) {
	param.ergebnis_array[ergebnis_ziel] = 
		log(param.ergebnis_array[zahl]);
}

void fi_sqrt::execute_item(int, ExecParam & param) {
	param.ergebnis_array[ergebnis_ziel] = 
		sqrt(param.ergebnis_array[zahl]);
}

void fi_abs::execute_item(int, ExecParam & param) {
	param.ergebnis_array[ergebnis_ziel] = 
		fabs(param.ergebnis_array[zahl]);
}

void fi_minus::execute_item(int, ExecParam & param) {
	param.ergebnis_array[ergebnis_ziel] = 
		-param.ergebnis_array[zahl];
}


void fi_varx::execute_item(int i, ExecParam & p) 
{
	double x = p.a+(p.b-p.a)*((double) i / (double) (p.y.size() - 1));
	p.ergebnis_array[ergebnis_ziel] = x;
}

void fi_pow::execute_item(int, ExecParam & param) {
	param.ergebnis_array[ergebnis_ziel] = 
		pow(param.ergebnis_array[zahl1],
		    param.ergebnis_array[zahl2]);
}

void fi_y::execute_item(int, ExecParam & p) 
{
  int j = (int) rint((p.ergebnis_array[zahl] - p.a)/(p.b-p.a)*
		     (double) (p.y.size() - 1));
  if (j < 0) {
    j = 0;
  } else if (j >= (int)p.y.size()) {
    j = p.y.size()-1;
  }
  
  p.ergebnis_array[ergebnis_ziel] = p.y[j];
}

void fi_dy::execute_item(int, ExecParam & p) 
{
  int size = p.y.size() - 1;
  double j = (p.ergebnis_array[zahl] - p.a)/(p.b-p.a)*
    (double) size;
  
  if ((j < 0) || (j > size)) {
    p.ergebnis_array[ergebnis_ziel] = 0;
  } else if (j == 0) {
    p.ergebnis_array[ergebnis_ziel] = (p.y[1] - p.y[0])
      * size / (p.b - p.a);
  } else if (j == size) {
    p.ergebnis_array[ergebnis_ziel] = (p.y[size] - p.y[size - 1])
      * size / (p.b - p.a);
  } else if (double(int(j)) == j) {
    p.ergebnis_array[ergebnis_ziel] = 
	    0.5 * (p.y[int(j+1)] - p.y[int(j-1)]) 
      * size / (p.b - p.a);
  }  else {	  
    p.ergebnis_array[ergebnis_ziel] = (p.y[int(j+1)] - p.y[int(j)])
      * size / (p.b - p.a);
  }
#ifdef DEBUG
  cout << "Steigung an " << p.ergebnis_array[zahl] << ":" <<
	  p.ergebnis_array[ergebnis_ziel] << endl;
#endif
}

void fi_gerade::execute_item(int j, ExecParam & p)
{ 
	double dy;
	int size = p.y.size() - 1;

	if (j == 0) {
		dy = (p.y[1] - p.y[0]) * size / (p.b - p.a);
	} else if (j == size) {
		dy = (p.y[size] - p.y[size - 1]) * size / (p.b - p.a);
	} else if (double(int(j)) == j) {
		dy = 0.5 * (p.y[int(j+1)] - p.y[int(j-1)]) 
			* size / (p.b - p.a);
	}

	p.ergebnis_array[ergebnis_ziel] = sqrt(dy*dy+1);
}


void fi_brachistochrone::execute_item(int j, ExecParam & p)
{ 
	double y = p.y[j];
	double dy;
	int size = p.y.size() - 1;

	if (j == 0) {
		dy = (p.y[1] - p.y[0]) * size / (p.b - p.a);
	} else if (j == size) {
		dy = (p.y[size] - p.y[size - 1]) * size / (p.b - p.a);
	} else if (double(int(j)) == j) {
		dy = 0.5 * (p.y[int(j+1)] - p.y[int(j-1)]) 
			* size / (p.b - p.a);
	}

	p.ergebnis_array[ergebnis_ziel] = sqrt((dy*dy+1)/(-y));
}

void fi_seifenhaut::execute_item(int j, ExecParam & p)
{ 
	double y = p.y[j];
	double dy;
	int size = p.y.size() - 1;

	if (j == 0) {
		dy = (p.y[1] - p.y[0]) * size / (p.b - p.a);
	} else if (j == size) {
		dy = (p.y[size] - p.y[size - 1]) * size / (p.b - p.a);
	} else if (double(int(j)) == j) {
		dy = 0.5 * (p.y[int(j+1)] - p.y[int(j-1)]) 
			* size / (p.b - p.a);
	}

	p.ergebnis_array[ergebnis_ziel] = y*sqrt(dy*dy+1);
}
