#ifndef FUNKTIONAL_H
#include <math.h>
#include <stdlib.h>
#include <list>
#include <vector>

class FunktionalItem {
protected:
  class ExecParam {
  public:
    double a;
    double b;
    vector<double> &y;
    vector<double> &ergebnis_array;
    
    ExecParam(double _a, double _b, vector<double> &_y,
	      vector<double> &_ergebnis_array) 
      : a(_a), b(_b), y(_y), ergebnis_array(_ergebnis_array) {};
  };
  
  int ergebnis_ziel;
public:
  virtual ~FunktionalItem();	
  virtual void execute_item(int i, ExecParam &param) = 0;
  
  void set_ziel(int zaehler) { ergebnis_ziel = zaehler; };
};

class Funktional {
  typedef list<FunktionalItem *> FList;
  FList f_liste;
  int zaehler;
  vector<double> ergebnis_array;
public:
  Funktional();
  ~Funktional();
  void initialize();
  int additem(FunktionalItem *item);
  double execute(double a, double b, vector<double> &y);
  
  char *parse(char *string);
private:
  Funktional(const Funktional &);
};

// ########################################################################
// Ab hier nur noch die langweiligen FunktionalItems-Klassendeklarationen
// ########################################################################

class fi_konst : public FunktionalItem {
	double zahl;
public:
  fi_konst(double z) : FunktionalItem()
    { zahl = z; }
  ~fi_konst() {};
  void execute_item(int i, ExecParam &param); 
};

class fi_add : public FunktionalItem {
  int zahl1;
  int zahl2;
public:
  fi_add(int z1, int z2) : FunktionalItem()
    { zahl1 = z1; zahl2 = z2; }
  ~fi_add() {};
  void execute_item(int i, ExecParam &param);
};

class fi_sub : public FunktionalItem {
  int zahl1;
  int zahl2;
public:
  fi_sub(int z1, int z2) : FunktionalItem()
    { zahl1 = z1; zahl2 = z2; }
  ~fi_sub() {};
  void execute_item(int i, ExecParam &param);
};

class fi_mul : public FunktionalItem {
  int zahl1;
  int zahl2;
public:
  fi_mul(int z1, int z2) : FunktionalItem()
    { zahl1 = z1; zahl2 = z2; }
  ~fi_mul() {};
  void execute_item(int i, ExecParam &param);
};

class fi_div : public FunktionalItem {
  int zahl1;
  int zahl2;
public:
  fi_div(int z1, int z2) : FunktionalItem()
    { zahl1 = z1; zahl2 = z2; }
  ~fi_div() {};
  void execute_item(int i, ExecParam &param);
};

class fi_sin : public FunktionalItem {
  int zahl;
public:
  fi_sin(int z) : FunktionalItem()
    { zahl = z; }
  ~fi_sin() {};
  void execute_item(int i, ExecParam &param);
};

class fi_cos : public FunktionalItem {
  int zahl;
public:
  fi_cos(int z) : FunktionalItem()
    { zahl = z; }
  ~fi_cos() {};
  void execute_item(int i, ExecParam &param);
};

class fi_tan : public FunktionalItem {
  int zahl;
public:
  fi_tan(int z) : FunktionalItem()
    { zahl = z; }
  ~fi_tan() {};
  void execute_item(int i, ExecParam &param);
};

class fi_exp : public FunktionalItem {
  int zahl;
public:
  fi_exp(int z) : FunktionalItem()
    { zahl = z; }
  ~fi_exp() {};
  void execute_item(int i, ExecParam &param);
};

class fi_ln : public FunktionalItem {
  int zahl;
public:
  fi_ln(int z) : FunktionalItem()
    { zahl = z; }
  ~fi_ln() {};
  void execute_item(int i, ExecParam &param);
};

class fi_sqrt : public FunktionalItem {
  int zahl;
public:
  fi_sqrt(int z) : FunktionalItem()
    { zahl = z; }
  ~fi_sqrt() {};
  void execute_item(int i, ExecParam &param);
};

class fi_abs : public FunktionalItem {
  int zahl;
public:
  fi_abs(int z) : FunktionalItem()
    { zahl = z; }
  ~fi_abs() {};
  void execute_item(int i, ExecParam &param);
};

class fi_minus : public FunktionalItem {
  int zahl;
public:
  fi_minus(int z) : FunktionalItem() {
    zahl = z;
  }
  ~fi_minus() {};
  void execute_item(int i, ExecParam &param);
};

class fi_varx : public FunktionalItem {
public:
  fi_varx() : FunktionalItem() {	}
  ~fi_varx() {};
  void execute_item(int i, ExecParam &param);
};

class fi_y : public FunktionalItem {
  int zahl;
public:
  fi_y(int z) : FunktionalItem() {
    zahl = z;
  }
  
  ~fi_y() {};
  void execute_item(int i, ExecParam &param);
};

class fi_gerade : public FunktionalItem {
public:
  fi_gerade() : FunktionalItem() { }
  
  ~fi_gerade() {};
  void execute_item(int i, ExecParam &param);
};

class fi_brachistochrone : public FunktionalItem {
public:
  fi_brachistochrone() : FunktionalItem() { }
  
  ~fi_brachistochrone() {};
  void execute_item(int i, ExecParam &param);
};

class fi_seifenhaut : public FunktionalItem {
public:
  fi_seifenhaut() : FunktionalItem() { }
  
  ~fi_seifenhaut() {};
  void execute_item(int i, ExecParam &param);
};

class fi_dy : public FunktionalItem {
  int zahl;
public:
  fi_dy(int z) : FunktionalItem() {
    zahl = z;
  }
  ~fi_dy() {};
  void execute_item(int i, ExecParam &param);
};


class fi_pow : public FunktionalItem {
  int zahl1;
  int zahl2;
public:
  fi_pow(int z1, int z2) : FunktionalItem() {
    zahl1 = z1; zahl2 = z2;
  }
  ~fi_pow() {};
  void execute_item(int i, ExecParam &param);
};

#define FUNKTIONAL_H
#endif
