#!/bin/sh
# the next line restarts using wish \
exec wish8.0 "$0" "$@"

#############################################################################

set solve_font "Utopia 48"

set sample_x 5
set sample_y 13
set window(x1) 0
set window(x2) 1
set window(y1) -1
set window(y2) 2

set points "0 1 1 1 1"

set solvetype 1
set funktionaltyp Gerade

set interpolation_points 3
set tcl_precision 17

#############################################################################

load [file dirname $argv0]/solve.so

#############################################################################


proc center_toplevel {w} {
    wm withdraw $w
    update idletasks
    set x [expr [winfo screenwidth $w]/2 - [winfo reqwidth $w]/2]
    set y [expr [winfo screenheight $w]/2 - [winfo reqheight $w]/2]
    wm geom $w +$x+$y
    wm deiconify $w
}

proc create_scrollbar {w var text breite} {    
    label $w.l -text $text
    entry $w.e -width 3 -textvariable $var
    scale $w.s -orient horiz -from 2 -to $breite -variable $var -showvalue 0
    pack $w.l -side left
    pack $w.s -side left -fill x -expand yes
    pack $w.e -side right
}

proc create_entries {w var1 var2 text} {
    label $w.l -text "$text: ("
    label $w.l2 -text "/"
    label $w.l3 -text ")"
    entry $w.e1 -width 3 -textvariable $var1
    entry $w.e2 -width 3 -textvariable $var2

    pack $w.l $w.e1 $w.l2 $w.e2 $w.l3 -side left    
}

proc solve_klicked {} {
    .scherz.b configure -text "42" -command "destroy .scherz"
    center_toplevel .scherz
}

#############################################################################
#############################################################################

set w ""

frame $w.menu -relief raised -bd 2
pack $w.menu -side top -fill x

#############################################################################
set m $w.menu.datei.m
menubutton $w.menu.datei -text "Datei" -menu $m -underline 0
menu $m -tearoff 0
$m add command -label "Neu" -underline 0 -command "loesche_ergebnis"
$m add command -label "ffnen" -underline 1 -command "lade_ergebnis"
$m add separator
$m add command -label "Speichern" -underline 0 -command "speichere_ergebnis"
$m add command -label "Speichern unter..." -underline 10 \
	-command "speichere_ergebnis_unter"
$m add separator
$m add command -label "Ende" -underline 0 -command "destroy ."

#############################################################################
set m $w.menu.optionen.m
menubutton $w.menu.optionen -text "Optionen" -menu $m -underline 0
menu $m -tearoff 0
$m add radio -label "Solve-mode: Brute-Force" -variable solvetype -value 0
$m add radio -label "Solve-mode: Optimize" -variable solvetype -value 1
$m add radio -label "Solve-mode: Optimize (Single Step)" \
	-variable solvetype -value 2
$m add separator
$m add radio -label "Interpolations-Punkte: 1" \
	-variable interpolation_points -value 1
$m add radio -label "Interpolations-Punkte: 3" \
	-variable interpolation_points -value 3
$m add radio -label "Interpolations-Punkte: 5" \
	-variable interpolation_points -value 5
$m add radio -label "Interpolations-Punkte: 7" \
	-variable interpolation_points -value 7

#############################################################################
set m $w.menu.hilfe.m
menubutton $w.menu.hilfe -text "Hilfe" -menu $m -underline 0
menu $m -tearoff 0
$m add command -label "Hilfe ist was fr Warmduscher..." -underline 0 
$m add separator
$m add command -label "Info" -underline 0 

pack    $w.menu.datei $w.menu.optionen -side left
pack    $w.menu.hilfe -side right
#############################################################################
frame .ergebnis -relief raised -background "Black"
frame .buttonbar -relief sunken

pack .ergebnis -side top -fill both -expand yes
pack .buttonbar -side bottom -fill x 

set w .buttonbar

button $w.solve -text "Solve" -font $solve_font -command "solve_it"
pack $w.solve -side left -fill both -expand yes

frame $w.sample
pack $w.sample -side left -fill both -expand yes

frame $w.sample.x
pack $w.sample.x -side top -fill x -expand yes
frame $w.sample.y
pack $w.sample.y -side top -fill x -expand yes

create_scrollbar $w.sample.x sample_x "Sample X:" 101
create_scrollbar $w.sample.y sample_y "Sample Y:" 1001

bind $w.sample.x.e <FocusOut> "resize_points"
bind $w.sample.x.s <Leave> "resize_points"
bind $w.sample.y.e <FocusOut> "korrigiere_punkte"
bind $w.sample.y.s <Leave> "korrigiere_punkte"

frame $w.punkte
pack $w.punkte -side left -fill y -expand yes

frame $w.punkte.a
pack $w.punkte.a -side top -fill x -expand yes
frame $w.punkte.b
pack $w.punkte.b -side top -fill x -expand yes

create_entries $w.punkte.a window(x1) window(x2) "X-Range"
create_entries $w.punkte.b window(y1) window(y2) "Y-Range"

bind $w.punkte.a.e1 <FocusOut> "korrigiere_punkte"
bind $w.punkte.a.e2 <FocusOut> "korrigiere_punkte"
bind $w.punkte.b.e1 <FocusOut> "korrigiere_punkte"
bind $w.punkte.b.e2 <FocusOut> "korrigiere_punkte"


frame $w.funktional
pack $w.funktional -side right -fill both -expand yes

frame $w.funktional.b
pack $w.funktional.b -side top -fill x

text $w.funktional.t -height 2 -width 40
pack $w.funktional.t -expand yes -fill both

set m [tk_optionMenu $w.funktional.b.options funktionaltyp \
	Custom... Gerade Brachistochrone Seifenhaut]

foreach i "1 2 3" {
    $m entryconfigure $i -command set_funktional
}
pack $w.funktional.b.options -side top -fill both

proc set_funktional {} {
    global funktionaltyp

    switch -exact -- $funktionaltyp {
	"Custom..." { }
	"Gerade" { 
	    .buttonbar.funktional.t delete 0.0 end
	    .buttonbar.funktional.t insert 0.0 "gerade()\n# sqrt(y'(x)^2+1)"
	}
	"Brachistochrone" { 
	    .buttonbar.funktional.t delete 0.0 end
	    .buttonbar.funktional.t insert 0.0 \
		    "brachistochrone()\n# sqrt((y'(x)^2+1)/(-20*y(x)))"
	}
	"Seifenhaut" { 
	    .buttonbar.funktional.t delete 0.0 end
	    .buttonbar.funktional.t insert 0.0 \
		    "seifenhaut()\n# y(x)*sqrt(y'(x)^2 + 1)"
	}
    }

}

#############################################################################

wm title . "Numerische Variationsrechnung, z.dt. Solve"
tkwait visibility .
#wm geometry . [winfo screenwidth $w]x[winfo screenheight $w]+0+0

#############################################################################
toplevel .scherz

wm title .scherz "Probleme?"
wm transient .scherz .

button .scherz.b -text "Solve!" -font $solve_font -command solve_klicked
pack .scherz.b -expand yes -fill both

update idletasks

center_toplevel .scherz
grab .scherz
tkwait window .scherz

#############################################################################
# Real work!
#############################################################################

proc screen_to_points {px py} {
    global window
    global points
    global sample_y

    set width [winfo width .ergebnis.c]
    set height [winfo height .ergebnis.c]


    set yscale [expr (abs($window(y1)) > abs($window(y2))) ? \
	    abs($window(y1)) : abs($window(y2))]
    set yscale [expr (($height/2)-12.0)/$yscale]

    set xdelta [expr (floor($width)-12.0)/([llength $points].0-1.0)]
    set x 6.0

    set pindex [expr round(($px-$x)/$xdelta)]
    set realpy [expr (($height/2-$py)/$yscale)]
    set grid [expr (floor($window(y2))-floor($window(y1)))/\
	    (floor($sample_y)-1.0)]
    set realpy [expr $grid*round($realpy / $grid)]
    if {$realpy < $window(y1)} {
	set realpy $window(y1)
    } elseif {$realpy > $window(y2)} {
	set realpy $window(y2)
    }

    return "$pindex $realpy [expr $px-$x]"
}


proc solve_it {} {
    global window
    global sample_x
    global sample_y
    global points
    global solvetype
    global interpolation_points

    set f "[.buttonbar.funktional.t get 0.0 end]\n"

    regsub -all -- "#.*\n" $f "" f

    solve setfunktional $f
    solve setwindow $window(x1) $window(x2) $window(y1) $window(y2)
    solve setsamples $sample_x $sample_y
    solve settype $solvetype
    solve setipoints $interpolation_points

    set points [eval solve execute $points]

    display_points
}

proc display_points {} {
    global points
    global window
   
    set c .ergebnis.c

    set width [winfo width .ergebnis.c]
    set height [winfo height .ergebnis.c]

    set xdelta [expr ($width.0-12.0)/([llength $points].0-1.0)]
    set x 6.0

    set yscale [expr (abs($window(y1)) > abs($window(y2))) ? \
	    abs($window(y1)) : abs($window(y2))]
    set yscale [expr (($height/2)-12)/$yscale]

    set rpoints ""
    foreach point $points {
	if {$point < $window(y1)} {
	    set point $window(y1)
	} elseif {$point > $window(y2)} {
	    set point $window(y2)
	}
	lappend rpoints $point
    }
    set points $rpoints

    $c delete point
    $c delete line

    set xold -1
    set yold -1

    set item [$c create line 0 [expr $height/2] $width [expr $height/2] \
	    -fill blue]
    $c addtag line withtag $item

    foreach point $points {	
	set y [expr $height/2-$point*$yscale]
	if {$xold != -1} {
	    set item [$c create line $xold $yold $x $y -fill green]
	    $c addtag line withtag $item
	}
	set xold $x
	set yold $y
	set x [expr $x+$xdelta]
    }

    set x 6.0

    foreach point $points {	
	set y [expr $height/2-$point*$yscale]
	set item [$c create oval [expr $x-6] [expr $y-6] \
		[expr $x+6] [expr $y+6] -width 1 -outline black \
		-fill SkyBlue2]
	$c addtag point withtag $item
	set x [expr $x+$xdelta]
    }
}

proc resize_points {} {
    global points
    global sample_x
    global window

    if {[llength $points] > $sample_x} {
	set points [lreplace $points $sample_x end]
	display_points
    } elseif {[llength $points] < $sample_x} {
	set val $window(y1)
	for {set i [llength $points]} {$i < $sample_x} {incr i} {
	    lappend points $val
	}
	display_points
    }  

}

proc korrigiere_punkte {} {
    global points
    global sample_y
    global window

    set rval ""
    set grid [expr (floor($window(y2))-floor($window(y1)))/\
	    (floor($sample_y)-1.0)]

    foreach realpy $points {
	set realpy [expr $grid*round($realpy / $grid)]
	if {$realpy < $window(y1)} {
	    set realpy $window(y1)
	} elseif {$realpy > $window(y2)} {
	    set realpy $window(y2)
	}
	lappend rval $realpy
    }
    set points $rval
    display_points
}

# plotDown --
# This procedure is invoked when the mouse is pressed over one of the
# data points.  It sets up state to allow the point to be dragged.
#
# Arguments:
# w -		The canvas window.
# x, y -	The coordinates of the mouse press.

proc plotDown {w x y} {
    global plot
    $w dtag selected
    $w addtag selected withtag current
    $w raise current
    set plot(lastX) $x
    set plot(lastY) $y
}

# plotMove --
# This procedure is invoked during mouse motion events.  It drags the
# current item.
#
# Arguments:
# w -		The canvas window.
# x, y -	The coordinates of the mouse.

proc plotMove {w x y} {
    global plot
    $w move selected 0 [expr $y-$plot(lastY)]
    set plot(lastX) $x
    set plot(lastY) $y

}

proc plotRelease {w} {
    global points
    global sample_y
    global window

    set pcoord [$w bbox selected]

    set px [expr ([lindex $pcoord 2]+[lindex $pcoord 0])/2]
    set py [expr ([lindex $pcoord 3]+[lindex $pcoord 1])/2]

    set tmp [screen_to_points $px $py]
    set pindex [lindex $tmp 0]
    set realpy [lindex $tmp 1]

    set points [lreplace $points $pindex $pindex $realpy]

    $w dtag selected

    display_points
}

proc plotEnter {c x y} {
    global points
    global window
    global sample_x

    set pcoord [$c bbox current]

    set px [expr ([lindex $pcoord 2]+[lindex $pcoord 0])/2]
    set py [expr ([lindex $pcoord 3]+[lindex $pcoord 1])/2]

    set tmp [screen_to_points $px $py]
    set pindex [lindex $tmp 0]

    $c itemconfig current -fill red

    .pdisplay entryconfigure 0 \
	   -label "[expr $pindex.0*($window(x2)-($window(x1)))/($sample_x-1)] \
	    [lindex $points $pindex]"

    .pdisplay post [expr [winfo rootx .ergebnis.c]+$x+10] \
	    [expr [winfo rooty .ergebnis.c]+$y+10]
}

proc plotLeave {c x y} {
    .pdisplay unpost
    $c itemconfig current -fill SkyBlue2
}

#############################################################################

menu .pdisplay -tearoff 0
.pdisplay add command -label "Test"

canvas .ergebnis.c -background Black
pack .ergebnis.c -expand yes -fill both

set c .ergebnis.c

$c bind point <Any-Enter> "plotEnter $c %x %y"
$c bind point <Any-Leave> "plotLeave $c %x %y"
$c bind point <1> "plotDown $c %x %y"
$c bind point <ButtonRelease-1> "plotRelease $c"
bind $c <B1-Motion> "plotMove $c %x %y"
bind . <Configure> display_points
bind $c <Leave> ".pdisplay unpost"

set plot(lastX) 0
set plot(lastY) 0

update idletasks

display_points
set_funktional

